@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -196,6 +196,11 @@ string AstNode::prettyName(const string& namein) VL_PURE {
                 pos += 7;
                 continue;
             }
+            if (0 == std::strncmp(pos, "__Viftop", 8)) {
+                pretty += "";
+                pos += 8;
+                continue;
+            }
             if (pos[0] == '_' && pos[1] == '_' && pos[2] == '0' && std::isxdigit(pos[3])
                 && std::isxdigit(pos[4])) {
                 char value = 0;
@@ -222,12 +227,14 @@ string AstNode::vpiName(const string& namein) {
     // This is slightly different from prettyName, in that when we encounter escaped characters,
     // we change that identifier to an escaped identifier, wrapping it with '\' and ' '
     // as specified in LRM 23.6
+    string name = namein;
+    if (0 == namein.substr(0, 7).compare("__SYM__")) { name = namein.substr(7); }
     string pretty;
-    pretty.reserve(namein.length());
+    pretty.reserve(name.length());
     bool inEscapedIdent = false;
     int lastIdent = 0;
 
-    for (const char* pos = namein.c_str(); *pos;) {
+    for (const char* pos = name.c_str(); *pos;) {
         char specialChar = 0;
         if (pos[0] == '-' && pos[1] == '>') {  // ->
             specialChar = '.';
@@ -795,13 +802,7 @@ void AstNode::swapWith(AstNode* bp) {
 
 AstNode* AstNode::cloneTreeIter(bool needPure) {
     // private: Clone single node and children
-    if (VL_UNLIKELY(needPure && !isPure())) {
-        this->v3warn(SIDEEFFECT,
-                     "Expression side effect may be mishandled\n"
-                         << this->warnMore()
-                         << "... Suggest use a temporary variable in place of this expression");
-        // this->v3fatalSrc("cloneTreePure debug backtrace");  // Comment in to debug where caused
-    }
+    if (needPure) purityCheck();
     AstNode* const newp = this->clone();
     if (this->m_op1p) newp->op1p(this->m_op1p->cloneTreeIterList(needPure));
     if (this->m_op2p) newp->op2p(this->m_op2p->cloneTreeIterList(needPure));
@@ -848,6 +849,16 @@ AstNode* AstNode::cloneTree(bool cloneNextLink, bool needPure) {
     return newp;
 }
 
+void AstNode::purityCheck() {
+    if (VL_UNLIKELY(!isPure())) {
+        this->v3warn(SIDEEFFECT,
+                     "Expression side effect may be mishandled\n"
+                         << this->warnMore()
+                         << "... Suggest use a temporary variable in place of this expression");
+        // this->v3fatalSrc("cloneTreePure debug backtrace");  // Comment in to debug where caused
+    }
+}
+
 //======================================================================
 // Delete
 
@@ -1101,7 +1112,7 @@ bool AstNode::sameTreeIter(const AstNode* node1p, const AstNode* node2p, bool ig
         (!node1p->dtypep() && !node2p->dtypep()) || (node1p->dtypep() && node2p->dtypep()), node1p,
         "Comparison of a node with dtypep() with a node without dtypep()\n-node2=" << node2p);
     if (node1p->dtypep() && !node1p->dtypep()->similarDType(node2p->dtypep())) return false;
-    if (!node1p->same(node2p) || (gateOnly && !node1p->isGateOptimizable())) return false;
+    if (!node1p->sameNode(node2p) || (gateOnly && !node1p->isGateOptimizable())) return false;
     return (sameTreeIter(node1p->m_op1p, node2p->m_op1p, false, gateOnly)
             && sameTreeIter(node1p->m_op2p, node2p->m_op2p, false, gateOnly)
             && sameTreeIter(node1p->m_op3p, node2p->m_op3p, false, gateOnly)
@@ -1286,10 +1297,17 @@ void AstNode::dumpPtrs(std::ostream& os) const {
 void AstNode::dumpTree(std::ostream& os, const string& indent, int maxDepth) const {
     static int s_debugFileline = v3Global.opt.debugSrcLevel("fileline");  // --debugi-fileline 9
     os << indent << " " << this << '\n';
+    if (VN_DELETED(this)) return;
     if (debug() > 8) {
         os << indent << "     ";
         dumpPtrs(os);
     }
+    if (VN_DELETED(op1p()) || VN_DELETED(op2p())  // LCOV_EXCL_START
+        || VN_DELETED(op3p()) || VN_DELETED(op4p())) {
+        os << indent << "1/2/3/4: %E-0x1/deleted! node " << cvtToHex(this)
+           << endl;  // endl intentional to do flush
+        return;
+    }  // LCOV_EXCL_STOP
     if (s_debugFileline >= 9) os << fileline()->warnContextSecondary();
     if (maxDepth == 1) {
         if (op1p() || op2p() || op3p() || op4p()) os << indent << "1: ...(maxDepth)\n";
@@ -1423,8 +1441,13 @@ string AstNode::instanceStr() const {
     return "";
 }
 void AstNode::v3errorEnd(std::ostringstream& str) const VL_RELEASE(V3Error::s().m_mutex) {
+    // Don't look for instance name when warning is disabled.
+    // In case of large number of warnings, this can
+    // take significant amount of time
+    const string instanceStrExtra
+        = m_fileline->warnIsOff(V3Error::s().errorCode()) ? "" : instanceStr();
     if (!m_fileline) {
-        V3Error::v3errorEnd(str, instanceStr());
+        V3Error::v3errorEnd(str, instanceStrExtra);
     } else {
         std::ostringstream nsstr;
         nsstr << str.str();
@@ -1434,11 +1457,7 @@ void AstNode::v3errorEnd(std::ostringstream& str) const VL_RELEASE(V3Error::s().
             const_cast<AstNode*>(this)->dump(nsstr);
             nsstr << endl;
         }
-        // Don't look for instance name when warning is disabled.
-        // In case of large number of warnings, this can
-        // take significant amount of time
-        m_fileline->v3errorEnd(
-            nsstr, m_fileline->warnIsOff(V3Error::s().errorCode()) ? "" : instanceStr());
+        m_fileline->v3errorEnd(nsstr, instanceStrExtra);
     }
 }
 void AstNode::v3errorEndFatal(std::ostringstream& str) const VL_RELEASE(V3Error::s().m_mutex) {
