@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -54,8 +54,10 @@ public:
 //######################################################################
 
 class BeginVisitor final : public VNVisitor {
-    // STATE
+    // STATE - across all visitors
     BeginState* const m_statep;  // Current global state
+
+    // STATE - for current visit position (use VL_RESTORER)
     AstNodeModule* m_modp = nullptr;  // Current module
     AstNodeFTask* m_ftaskp = nullptr;  // Current function/task
     AstNode* m_liftedp = nullptr;  // Local  nodes we are lifting into m_ftaskp
@@ -145,10 +147,20 @@ class BeginVisitor final : public VNVisitor {
     }
     void visit(AstNodeModule* nodep) override {
         VL_RESTORER(m_modp);
-        {
-            m_modp = nodep;
-            iterateChildren(nodep);
+        m_modp = nodep;
+        // Rename it (e.g. class under a generate)
+        if (m_unnamedScope != "") {
+            nodep->name(dot(m_unnamedScope, nodep->name()));
+            UINFO(8, "     rename to " << nodep->name() << endl);
+            m_statep->userMarkChanged(nodep);
         }
+        VL_RESTORER(m_displayScope);
+        VL_RESTORER(m_namedScope);
+        VL_RESTORER(m_unnamedScope);
+        m_displayScope = "";
+        m_namedScope = "";
+        m_unnamedScope = "";
+        iterateChildren(nodep);
     }
     void visit(AstNodeFTask* nodep) override {
         UINFO(8, "  " << nodep << endl);
@@ -164,29 +176,28 @@ class BeginVisitor final : public VNVisitor {
         // inside the function.
         // Process children
         VL_RESTORER(m_displayScope);
+        VL_RESTORER(m_ftaskp);
+        VL_RESTORER(m_liftedp);
         VL_RESTORER(m_namedScope);
         VL_RESTORER(m_unnamedScope);
-        {
-            m_displayScope = dot(m_displayScope, nodep->name());
-            m_namedScope = "";
-            m_unnamedScope = "";
-            m_ftaskp = nodep;
-            m_liftedp = nullptr;
-            iterateChildren(nodep);
-            nodep->foreach([&](AstInitialStatic* const initp) {
-                initp->unlinkFrBack();
-                m_ftaskp->addHereThisAsNext(initp);
-            });
-            if (m_liftedp) {
-                // Place lifted nodes at beginning of stmtsp, so Var nodes appear before referenced
-                if (AstNode* const stmtsp = nodep->stmtsp()) {
-                    stmtsp->unlinkFrBackWithNext();
-                    m_liftedp->addNext(stmtsp);
-                }
-                nodep->addStmtsp(m_liftedp);
-                m_liftedp = nullptr;
+        m_displayScope = dot(m_displayScope, nodep->name());
+        m_namedScope = "";
+        m_unnamedScope = "";
+        m_ftaskp = nodep;
+        m_liftedp = nullptr;
+        iterateChildren(nodep);
+        nodep->foreach([&](AstInitialStatic* const initp) {
+            initp->unlinkFrBack();
+            m_ftaskp->addHereThisAsNext(initp);
+        });
+        if (m_liftedp) {
+            // Place lifted nodes at beginning of stmtsp, so Var nodes appear before referenced
+            if (AstNode* const stmtsp = nodep->stmtsp()) {
+                stmtsp->unlinkFrBackWithNext();
+                m_liftedp->addNext(stmtsp);
             }
-            m_ftaskp = nullptr;
+            nodep->addStmtsp(m_liftedp);
+            m_liftedp = nullptr;
         }
     }
     void visit(AstBegin* nodep) override {
@@ -196,30 +207,28 @@ class BeginVisitor final : public VNVisitor {
         VL_RESTORER(m_namedScope);
         VL_RESTORER(m_unnamedScope);
         {
-            {
-                VL_RESTORER(m_keepBegins);
-                m_keepBegins = false;
-                dotNames(nodep, "__BEGIN__");
-            }
-            UASSERT_OBJ(!nodep->genforp(), nodep, "GENFORs should have been expanded earlier");
-
-            // Cleanup
-            if (m_keepBegins) {
-                nodep->name("");
-                return;
-            }
-            AstNode* addsp = nullptr;
-            if (AstNode* const stmtsp = nodep->stmtsp()) {
-                stmtsp->unlinkFrBackWithNext();
-                addsp = AstNode::addNext(addsp, stmtsp);
-            }
-            if (addsp) {
-                nodep->replaceWith(addsp);
-            } else {
-                nodep->unlinkFrBack();
-            }
-            VL_DO_DANGLING(pushDeletep(nodep), nodep);
+            VL_RESTORER(m_keepBegins);
+            m_keepBegins = false;
+            dotNames(nodep, "__BEGIN__");
         }
+        UASSERT_OBJ(!nodep->genforp(), nodep, "GENFORs should have been expanded earlier");
+
+        // Cleanup
+        if (m_keepBegins) {
+            nodep->name("");
+            return;
+        }
+        AstNode* addsp = nullptr;
+        if (AstNode* const stmtsp = nodep->stmtsp()) {
+            stmtsp->unlinkFrBackWithNext();
+            addsp = AstNode::addNext(addsp, stmtsp);
+        }
+        if (addsp) {
+            nodep->replaceWith(addsp);
+        } else {
+            nodep->unlinkFrBack();
+        }
+        VL_DO_DANGLING(pushDeletep(nodep), nodep);
     }
     void visit(AstVar* nodep) override {
         // If static variable, move it outside a function.
@@ -427,6 +436,7 @@ AstNode* V3Begin::convertToWhile(AstForeach* nodep) {
     AstNode* bodyPointp = new AstBegin{nodep->fileline(), "[EditWrapper]", nullptr};
     AstNode* newp = nullptr;
     AstNode* lastp = nodep;
+    AstVar* nestedIndexp = nullptr;
     // subfromp used to traverse each dimension of multi-d variable-sized unpacked array (queue,
     // dyn-arr and associative-arr)
     AstNodeExpr* subfromp = fromp->cloneTreePure(false);
@@ -459,8 +469,13 @@ AstNode* V3Begin::convertToWhile(AstForeach* nodep) {
                 }
             } else if (VN_IS(fromDtp, DynArrayDType) || VN_IS(fromDtp, QueueDType)) {
                 AstConst* const leftp = new AstConst{fl, 0};
-                AstNodeExpr* const rightp
-                    = new AstCMethodHard{fl, subfromp->cloneTreePure(false), "size"};
+                AstNodeExpr* const rightp = new AstCMethodHard{
+                    fl,
+                    VN_IS(subfromp->dtypep(), NodeArrayDType)
+                        ? new AstArraySel{fl, subfromp->cloneTreePure(false),
+                                          new AstVarRef{fl, nestedIndexp, VAccess::READ}}
+                        : subfromp->cloneTreePure(false),
+                    "size"};
                 AstVarRef* varRefp = new AstVarRef{fl, varp, VAccess::READ};
                 subfromp = new AstCMethodHard{fl, subfromp, "at", varRefp};
                 subfromp->dtypep(fromDtp);
@@ -511,6 +526,7 @@ AstNode* V3Begin::convertToWhile(AstForeach* nodep) {
             if (!newp) newp = loopp;
         }
         // Prep for next
+        nestedIndexp = varp;
         fromDtp = fromDtp->subDTypep();
     }
     // The parser validates we don't have "foreach (array[,,,])"
