@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -208,6 +208,7 @@ class DelayedVisitor final : public VNVisitor {
     // NODE STATE
     //  AstVar::user1()         -> bool.  Set true if already issued MULTIDRIVEN warning
     //  AstVarRef::user1()      -> bool.  Set true if target of NBA
+    //  AstAssignDly::user1()   -> bool.  Set true if already visited
     //  AstNodeModule::user1p() -> std::unorded_map<std::string, AstVar*> temp map via m_varMap
     //  AstVarScope::user1p()   -> VarScopeInfo via m_vscpInfo
     //  AstVarScope::user2p()   -> AstVarRef*: First write reference to the Variable
@@ -251,10 +252,11 @@ class DelayedVisitor final : public VNVisitor {
         const AstNodeDType* const dtypep = vscp->dtypep()->skipRefp();
         // Unpacked arrays
         if (const AstUnpackArrayDType* const uaDTypep = VN_CAST(dtypep, UnpackArrayDType)) {
+            // Basic underlying type of elements, if any.
+            AstBasicDType* const basicp = uaDTypep->basicp();
             // If used in a loop, we must have a dynamic commit queue. (Also works in suspendables)
             if (vscpInfo.m_inLoop) {
                 // Arrays with compound element types are currently not supported in loops
-                AstBasicDType* const basicp = uaDTypep->basicp();
                 if (!basicp
                     || !(basicp->isIntegralOrPacked()  //
                          || basicp->isDouble()  //
@@ -266,8 +268,12 @@ class DelayedVisitor final : public VNVisitor {
             }
             // In a suspendable of fork, we must use the unique flag scheme, TODO: why?
             if (vscpInfo.m_inSuspOrFork) return Scheme::FlagUnique;
-            // Otherwise use the shared flag scheme
-            return Scheme::FlagShared;
+            // Otherwise if an array of packed/basic elements, use the shared flag scheme
+            if (basicp) return Scheme::FlagShared;
+            // Finally fall back on the shadow variable scheme, e.g. for
+            // arrays of unpacked structs. This will be slow.
+            // TODO: generic LHS scheme as discussed in #5092
+            return Scheme::ShadowVar;
         }
 
         // In a suspendable of fork, we must use the unique flag scheme, TODO: why?
@@ -529,17 +535,17 @@ class DelayedVisitor final : public VNVisitor {
     }
 
     // Scheme::ValueQueuePartial/Scheme::ValueQueueWhole
-    template <bool Partial>
+    template <bool N_Partial>
     void prepareSchemeValueQueue(AstVarScope* vscp, VarScopeInfo& vscpInfo) {
-        UASSERT_OBJ(Partial ? vscpInfo.m_scheme == Scheme::ValueQueuePartial
-                            : vscpInfo.m_scheme == Scheme::ValueQueueWhole,
+        UASSERT_OBJ(N_Partial ? vscpInfo.m_scheme == Scheme::ValueQueuePartial
+                              : vscpInfo.m_scheme == Scheme::ValueQueueWhole,
                     vscp, "Inconsisten<t NBA s>cheme");
         FileLine* const flp = vscp->fileline();
         AstScope* const scopep = vscp->scopep();
 
         // Create the commit queue variable
         auto* const cqDTypep
-            = new AstNBACommitQueueDType{flp, vscp->dtypep()->skipRefp(), Partial};
+            = new AstNBACommitQueueDType{flp, vscp->dtypep()->skipRefp(), N_Partial};
         v3Global.rootp()->typeTablep()->addTypesp(cqDTypep);
         const std::string name = "__VdlyCommitQueue" + vscp->varp()->shortName();
         AstVarScope* const queueVscp = createTemp(flp, scopep, name, cqDTypep);
@@ -891,6 +897,7 @@ class DelayedVisitor final : public VNVisitor {
                 ifp->addThensp(newp);
             }
 
+            UASSERT_OBJ(m_activep, nodep, "No active to handle FireEvent");
             AstActive* const activep = new AstActive{flp, "nba-event", m_activep->sensesp()};
             m_activep->addNextHere(activep);
             activep->addStmtsp(prep);
@@ -903,6 +910,9 @@ class DelayedVisitor final : public VNVisitor {
         VL_DO_DANGLING(nodep->deleteTree(), nodep);
     }
     void visit(AstAssignDly* nodep) override {
+        // Prevent double processing due to AstExprStmt being moved before this node
+        if (nodep->user1SetOnce()) return;
+
         if (m_cfuncp) {
             if (!v3Global.rootp()->nbaEventp()) {
                 nodep->v3warn(
@@ -916,21 +926,31 @@ class DelayedVisitor final : public VNVisitor {
         UASSERT_OBJ(m_inSuspendableOrFork || m_activep->hasClocked(), nodep,
                     "<= assignment in non-clocked block, should have been converted in V3Active");
 
-        // Grab the reference to the target of the NBA
+        // Grab the reference to the target of the NBA, also lift ExprStmt statements on the LHS
         VL_RESTORER(m_currNbaLhsRefp);
         UASSERT_OBJ(!m_currNbaLhsRefp, nodep, "NBAs should not nest");
-        nodep->lhsp()->foreach([&](AstVarRef* nodep) {
-            // Ignore reads (e.g.: '_[*here*] <= _')
-            if (nodep->access().isReadOnly()) return;
-            // A RW ref on the LHS (e.g.: '_[preInc(*here*)] <= _') is asking for trouble at this
-            // point. These should be lowered in an earlier pass into sequenced temporaries.
-            UASSERT_OBJ(!nodep->access().isRW(), nodep, "RW ref on LHS of NBA");
-            // Multiple target variables
-            // (e.g.: '{*here*, *and here*} <= _',or '*here*[*and here* = _] <= _').
-            // These should be lowered in an earlier pass into sequenced statements.
-            UASSERT_OBJ(!m_currNbaLhsRefp, nodep, "Multiple Write refs on LHS of NBA");
-            // Hold on to it
-            m_currNbaLhsRefp = nodep;
+        nodep->lhsp()->foreach([&](AstNode* currp) {
+            if (AstExprStmt* const exprp = VN_CAST(currp, ExprStmt)) {
+                // Move statements before the NBA
+                nodep->addHereThisAsNext(exprp->stmtsp()->unlinkFrBackWithNext());
+                // Replace with result
+                currp->replaceWith(exprp->resultp()->unlinkFrBack());
+                // Get rid of the AstExprStmt
+                VL_DO_DANGLING(pushDeletep(currp), currp);
+            } else if (AstVarRef* const refp = VN_CAST(currp, VarRef)) {
+                // Ignore reads (e.g.: '_[*here*] <= _')
+                if (refp->access().isReadOnly()) return;
+                // A RW ref on the LHS (e.g.: '_[preInc(*here*)] <= _') is asking for trouble at
+                // this point. These should be lowered in an earlier pass into sequenced
+                // temporaries.
+                UASSERT_OBJ(!refp->access().isRW(), refp, "RW ref on LHS of NBA");
+                // Multiple target variables
+                // (e.g.: '{*here*, *and here*} <= _',or '*here*[*and here* = _] <= _').
+                // These should be lowered in an earlier pass into sequenced statements.
+                UASSERT_OBJ(!m_currNbaLhsRefp, refp, "Multiple Write refs on LHS of NBA");
+                // Hold on to it
+                m_currNbaLhsRefp = refp;
+            }
         });
         // The target variable of the NBA (there can only be one per NBA at this point)
         AstVarScope* const vscp = m_currNbaLhsRefp->varScopep();
