@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -47,6 +47,8 @@ class LinkResolveVisitor final : public VNVisitor {
     // Below state needs to be preserved between each module call.
     AstNodeModule* m_modp = nullptr;  // Current module
     AstClass* m_classp = nullptr;  // Class we're inside
+    string m_randcIllegalWhy;  // Why randc illegal
+    AstNode* m_randcIllegalp = nullptr;  // Node causing randc illegal
     AstNodeFTask* m_ftaskp = nullptr;  // Function or task we're inside
     AstNodeCoverOrAssert* m_assertp = nullptr;  // Current assertion
     int m_senitemCvtNum = 0;  // Temporary signal counter
@@ -63,19 +65,49 @@ class LinkResolveVisitor final : public VNVisitor {
         if (nodep->dead()) return;
         VL_RESTORER(m_modp);
         VL_RESTORER(m_senitemCvtNum);
-        {
-            m_modp = nodep;
-            m_senitemCvtNum = 0;
-            iterateChildren(nodep);
-        }
+        m_modp = nodep;
+        m_senitemCvtNum = 0;
+        iterateChildren(nodep);
     }
     void visit(AstClass* nodep) override {
         VL_RESTORER(m_classp);
-        {
-            m_classp = nodep;
-            iterateChildren(nodep);
-        }
+        m_classp = nodep;
+        iterateChildren(nodep);
     }
+    void visit(AstConstraint* nodep) override {
+        // V3LinkDot moved the isExternDef into the class, the extern proto was
+        // checked to exist, and now isn't needed
+        nodep->isExternDef(false);
+        if (nodep->isExternProto()) {
+            VL_DO_DANGLING(nodep->unlinkFrBack()->deleteTree(), nodep);
+            return;
+        }
+        iterateChildren(nodep);
+    }
+    void visit(AstConstraintBefore* nodep) override {
+        VL_RESTORER(m_randcIllegalWhy);
+        VL_RESTORER(m_randcIllegalp);
+        m_randcIllegalWhy = "'solve before' (IEEE 1800-2023 18.5.9)";
+        m_randcIllegalp = nodep;
+        iterateChildrenConst(nodep);
+    }
+    void visit(AstDist* nodep) override {
+        VL_RESTORER(m_randcIllegalWhy);
+        VL_RESTORER(m_randcIllegalp);
+        m_randcIllegalWhy = "'constraint dist' (IEEE 1800-2023 18.5.3)";
+        m_randcIllegalp = nodep;
+        iterateChildrenConst(nodep);
+    }
+    void visit(AstConstraintExpr* nodep) override {
+        VL_RESTORER(m_randcIllegalWhy);
+        VL_RESTORER(m_randcIllegalp);
+        if (nodep->isSoft()) {
+            m_randcIllegalWhy = "'constraint soft' (IEEE 1800-2023 18.5.13.1)";
+            m_randcIllegalp = nodep;
+        }
+        iterateChildrenConst(nodep);
+    }
+
     void visit(AstInitialAutomatic* nodep) override {
         iterateChildren(nodep);
         // Initial assignments under function/tasks can just be simple
@@ -104,13 +136,24 @@ class LinkResolveVisitor final : public VNVisitor {
     }
 
     void visit(AstNodeVarRef* nodep) override {
-        // VarRef: Resolve its reference
-        if (nodep->varp()) nodep->varp()->usedParam(true);
-        // TODO should look for where genvar is valid, but for now catch
-        // just gross errors of using genvar outside any generate
-        if (nodep->varp() && nodep->varp()->isGenVar() && !m_underGenFor) {
-            nodep->v3error("Genvar " << nodep->prettyNameQ()
-                                     << " used outside generate for loop (IEEE 1800-2023 27.4)");
+        if (nodep->varp()) {  // Else due to dead code, might not have var pointer
+            // VarRef: Resolve its reference
+            nodep->varp()->usedParam(true);
+            // TODO should look for where genvar is valid, but for now catch
+            // just gross errors of using genvar outside any generate
+            if (nodep->varp()->isGenVar() && !m_underGenFor) {
+                nodep->v3error("Genvar "
+                               << nodep->prettyNameQ()
+                               << " used outside generate for loop (IEEE 1800-2023 27.4)");
+            }
+            if (nodep->varp()->isRandC() && m_randcIllegalp) {
+                nodep->v3error("Randc variables not allowed in "
+                               << m_randcIllegalWhy << '\n'
+                               << nodep->warnContextPrimary() << '\n'
+                               << m_randcIllegalp->warnOther()
+                               << "... Location of restricting expression\n"
+                               << m_randcIllegalp->warnContextSecondary());
+            }
         }
         iterateChildren(nodep);
     }
@@ -178,7 +221,8 @@ class LinkResolveVisitor final : public VNVisitor {
             letp->user2(false);
             return;
         }
-        if (nodep->taskp() && (nodep->taskp()->dpiContext() || nodep->taskp()->dpiExport())) {
+        if (nodep->taskp() && !nodep->scopeNamep()
+            && (nodep->taskp()->dpiContext() || nodep->taskp()->dpiExport())) {
             nodep->scopeNamep(new AstScopeName{nodep->fileline(), false});
         }
     }
