@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -182,16 +182,11 @@ class SenExprBuilder final {
         case VEdgeType::ET_EVENT: {
             UASSERT_OBJ(v3Global.hasEvents(), senItemp, "Inconsistent");
             {
-                // If the event is fired, set up the clearing process
-                AstCMethodHard* const callp = new AstCMethodHard{flp, currp(), "isFired"};
-                callp->dtypeSetBit();
-                AstIf* const ifp = new AstIf{flp, callp};
-                m_postUpdates.push_back(ifp);
-
                 // Clear 'fired' state when done
+                // No need to check if the event was fired, we need the flag clear regardless
                 AstCMethodHard* const clearp = new AstCMethodHard{flp, currp(), "clearFired"};
                 clearp->dtypeSetVoid();
-                ifp->addThensp(clearp->makeStmt());
+                m_postUpdates.push_back(clearp->makeStmt());
             }
 
             // Get 'fired' state
